//
//  Copyright (C) 1994-2003  Seth Kurtzberg
//
// CQL++ This program is free software.  The copyright is reserved.
//
// The file COPYING included with the distribution specifies the license
// granted to users of CQL++.  You can also view the licensing terms
// at http://www.cql.com/copying.html
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See COPYING
// for more information
//
// If you did not receive the file COPYING which contains the terms of
// the license, and you do not have access to the world wide web, you
// can request a copy by the following email address:  support@cql.com.
// If you do not have access to email, you can obtain a copy by postal
// mail by writing to Machine Independent Software Corporation,
// 9663 East Voltaire Drive, Scottsdale, Arizona, 85260, USA
//


#include "CqlSqlInclude.h"


extern FILE *yyin;
extern int yydebug;
extern int debugActionFunctions;


void Parser::newStatement( void )
{
	std::cerr << "SQL> ";
	std::cerr.flush();
}


int main( int argc, char *argv[] )
{
	if( argc < 2 )
	{
		std::cerr << "interpreter {UserName} [InitialDirectoryName]"
				  << std:: endl;
		exit( -1 );
	}
	char *initialDirectory = const_cast<char*>("");
	if( argc == 3 )
		initialDirectory = argv[2];

	if( getenv( "CQL_DATA_TRACING" ) )
		dataTracing = 1;

	if( getenv( "CQL_PARSER_TRACING" ) )
		yydebug = 1;

	SqlHandle *dbsql;

	try
	{
		dbsql = new SqlHandle( 0, 0, initialDirectory );
	}
	catch( CqlException& ex )
	{
		std::cerr << ex;
		return 0;
	}

	//  yydebug = 1;
	//  debugActionFunctions = 1;


	dbSqlEnv->SetInteractiveMode();

	try
	{
		if( argc < 2 )
			dbsql->connect( "PUBLIC" );
		else
			dbsql->connect( argv[1] );
	}
	catch( CqlException& fme )
	{
		std::cerr << fme;
		return 0;
	}

	yyin = stdin;
	dbsql->parse()->setEcho();
	dbsql->parse()->newStatement();

	while( !dbsql->parse()->shuttingDown() )
	{
		dbsql->resetError();
#if !defined(__CQL_CLIENT_SERVER__)
		try
		{
#endif
			if( yydebug )
				std::cerr << "Calling yyparse..." << std::endl;

			yyparse( dbsql );
#if !defined(__CQL_CLIENT_SERVER__)
		}
		catch( CqlException& exc )
		{
			std::cerr << exc;
			break;
		}
#endif

		if( !dbsql->noError() )
		{
			std::cerr << "Error " << ((NI)dbsql->errorCode()) << ", file "
				<< dbsql->errorFile() << ", line "
				<< dbsql->errorLine() << ", OS error "
				<< dbsql->osError() << std::endl;
			dbsql->parse()->newStatement();
		}
	}

	dbsql->disconnect();
	std::cerr << "Interpreter exit" << std::endl;
	delete dbsql;
	delete dbSqlEnv;
	return 0;
}
